
// -------------------------------------------------------------------------------------
// This class uses the code copied over from "MSCP - Marcel's Simple Chess Program" 
// MSCP: marcelk@bitpit.net / Marcel van Kervinck / http://marcelk.net/mscp/
// Code stripped down and modified for my own needs: miko@mikoweb.de 8/2010
// -------------------------------------------------------------------------------------

// TODO: Add something to detect draw50

#include "stdafx.h"

/* 8 bits per square representing which directions a king can walk to */
const byte CMSCP::king_dirs[64] = {
          7,  31,  31,  31,  31,  31,  31,  28,
        199, 255, 255, 255, 255, 255, 255, 124,
        199, 255, 255, 255, 255, 255, 255, 124,
        199, 255, 255, 255, 255, 255, 255, 124,
        199, 255, 255, 255, 255, 255, 255, 124,
        199, 255, 255, 255, 255, 255, 255, 124,
        199, 255, 255, 255, 255, 255, 255, 124,
        193, 241, 241, 241, 241, 241, 241, 112,
		};

/* 8 bits per square representing which directions a knight can jump to */
const byte CMSCP::knight_dirs[64] = {
         6,  14,  46,  46,  46,  46,  44,  40,
         7,  15,  63,  63,  63,  63,  60,  56,
        71, 207, 255, 255, 255, 255, 252, 184,
        71, 207, 255, 255, 255, 255, 252, 184,
        71, 207, 255, 255, 255, 255, 252, 184,
        71, 207, 255, 255, 255, 255, 252, 184,
        67, 195, 243, 243, 243, 243, 240, 176,
        65, 193, 209, 209, 209, 209, 208, 144,
		};

const int CMSCP::prescore_piece_value[13] = {
		0,
		0, 9<<9, 5<<9, 3<<9, 3<<9, 1<<9,
		0, 9<<9, 5<<9, 3<<9, 3<<9, 1<<9
		};

const int CMSCP::center[64] = {
         0,  2,  3,  4,  4,  3,  2,  0,
         2,  3,  4,  5,  5,  4,  3,  2,
         3,  4,  5,  7,  7,  5,  4,  3,
         4,  5,  7, 10, 10,  7,  5,  4,
         4,  5,  7, 10, 10,  7,  5,  4,
         3,  4,  5,  7,  7,  5,  4,  3,
         2,  3,  4,  5,  5,  4,  3,  2,
         0,  2,  3,  4,  4,  3,  2,  0,
		 };

const int CMSCP::pawn_advance[64] = {
         0,  0, 2, 3, 5, 30, 50, 0,
         0,  0, 2, 3, 5, 30, 50, 0,
         0,  0, 2, 3, 5, 30, 50, 0,
         0, -5, 2, 8, 8, 30, 50, 0,
         0, -5, 2, 8, 8, 30, 50, 0,
         0,  0, 2, 3, 5, 30, 50, 0,
         0,  0, 2, 3, 5, 30, 50, 0,
         0,  0, 2, 3, 5, 30, 50, 0,
		 };

const int CMSCP::white_control[64] = {
         1,  1,  1,  1,  2,  2,  2,  2,
         1,  1,  1,  1,  2,  2,  2,  2,
         1,  1,  2,  2,  3,  3,  2,  2,
         1,  1,  2,  5,  6,  3,  2,  2,
         1,  1,  2,  5,  6,  3,  2,  2,
         1,  1,  2,  2,  3,  3,  2,  2,
         1,  1,  1,  1,  2,  2,  2,  2,
         1,  1,  1,  1,  2,  2,  2,  2,
		 };

// ---------------------------------------------------------------------------------
CMSCP::CMSCP(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::CMSCP called.");

	maxdepth = 4;
	m_bAbortCalculation = false;
	m_bIsCalculating = false;
	m_wcsCallbackFunction.clear();
	m_hThreadHandle = NULL;

	king_step[1<<0] =  1;
    king_step[1<<1] =  9;
    king_step[1<<2] =  8;
    king_step[1<<3] =  7;
    king_step[1<<4] = -1;
    king_step[1<<5] = -9;
    king_step[1<<6] = -8;
    king_step[1<<7] = -7;

    knight_jump[1<<0] =  -6;
    knight_jump[1<<1] =  10;
    knight_jump[1<<2] =  17;
    knight_jump[1<<3] =  15;
    knight_jump[1<<4] = -10;
    knight_jump[1<<5] =   6;
    knight_jump[1<<6] = -15;
    knight_jump[1<<7] = -17;

	castle[A1] = CASTLE_WHITE_QUEEN;
    castle[E1] = CASTLE_WHITE_KING | CASTLE_WHITE_QUEEN;
    castle[H1] = CASTLE_WHITE_KING;
    castle[A8] = CASTLE_BLACK_QUEEN;
    castle[E8] = CASTLE_BLACK_KING | CASTLE_BLACK_QUEEN;
    castle[H8] = CASTLE_BLACK_KING;

	rnd_seed = time(NULL);

	for (int i=0; i<sizeof(zobrist); i++) 
	{
		((byte*)zobrist)[i] = rnd() & 0xff;
    }

	 NewGame();
}

// ---------------------------------------------------------------------------------
CMSCP::~CMSCP(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::~CMSCP called.");
	CloseWorkerThread();
}

// ---------------------------------------------------------------------------------
void CMSCP::CloseWorkerThread(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::CloseWorkerThread called.");

	if (m_hThreadHandle) 
	{
		m_bAbortCalculation = true;
		while (m_bIsCalculating) Sleep(10);
		CloseHandle(m_hThreadHandle);
		m_hThreadHandle = NULL;
		GL_WriteDebugOut(L"CMSCP::CloseWorkerThread closed thread.");
	}
}

// ---------------------------------------------------------------------------------
int __cdecl CMSCP::cmp_move(const void *ap, const void *bp)
// ---------------------------------------------------------------------------------
{
        const struct move *a = (move*) ap;
        const struct move *b = (move*) bp;

        if (a->prescore < b->prescore) return -1;
        if (a->prescore > b->prescore) return 1;
        return a->imove - b->imove; /* this makes qsort deterministic */
}

// ---------------------------------------------------------------------------------
long CMSCP::rnd(void)
// ---------------------------------------------------------------------------------
{
        long r = rnd_seed;

        r = 16807 * (r % 127773L) - 2836 * (r / 127773L);
        if (r < 0) r += 0x7fffffffL;

        return rnd_seed = r;
}

/* Compute 32-bit Zobrist hash key. Normally 32 bits this is too small,
   but for MSCP's small searches it is OK */
// ---------------------------------------------------------------------------------
unsigned long CMSCP::compute_hash(void)
// ---------------------------------------------------------------------------------
{
        unsigned long hash = 0;
        int sq;

        for (sq=0; sq<64; sq++) {
                if (board[sq] != EMPTY) {
                        hash ^= zobrist[board[sq]-1][sq];
                }
        }

		return hash ^ WTM;
}

// ---------------------------------------------------------------------------------
void CMSCP::reset(void)
// ---------------------------------------------------------------------------------
{
        move_sp = move_stack;
        undo_sp = undo_stack;
        hash_stack[ply] = compute_hash();
}

// ---------------------------------------------------------------------------------
void CMSCP::atk_slide(int sq, byte dirs, struct side *s)
// ---------------------------------------------------------------------------------
{
        byte dir = 0;
        int to;

        dirs &= king_dirs[sq];
        do {
                dir -= dirs;
                dir &= dirs;
                to = sq;
                do {
                        to += king_step[dir];
                        s->attack[to] += 1;
                        if (board[to] != EMPTY) break;
                } while (dir & king_dirs[to]);
        } while (dirs -= dir);
}

// ---------------------------------------------------------------------------------
void CMSCP::compute_attacks(void)
// ---------------------------------------------------------------------------------
{
        int sq, to, pc;
        byte dir, dirs;

        memset(&white, 0, sizeof white);
        memset(&black, 0, sizeof black);

        myfriend = WTM ? &white : &black;
        enemy = WTM ? &black : &white;

        for (sq=0; sq<64; sq++) {
                pc = board[sq];
                if (pc == EMPTY) continue;

                switch (pc) {
                case WHITE_KING:
                        dir = 0;
                        white.king = sq;
                        dirs = king_dirs[sq];
                        do {
                                dir -= dirs;
                                dir &= dirs;
                                to = sq + king_step[dir];
                                white.attack[to] += 1;
                        } while (dirs -= dir);
                        break;

                case BLACK_KING:
                        dir = 0;
                        black.king = sq;
                        dirs = king_dirs[sq];
                        do {
                                dir -= dirs;
                                dir &= dirs;
                                to = sq + king_step[dir];
                                black.attack[to] += 1;
                        } while (dirs -= dir);
                        break;

                case WHITE_QUEEN:
                        atk_slide(sq, ATK_SLIDER, &white);
                        break;

                case BLACK_QUEEN:
                        atk_slide(sq, ATK_SLIDER, &black);
                        break;

                case WHITE_ROOK:
                        atk_slide(sq, ATK_ORTHOGONAL, &white);
                        break;

                case BLACK_ROOK:
                        atk_slide(sq, ATK_ORTHOGONAL, &black);
                        break;

                case WHITE_BISHOP:
                        atk_slide(sq, ATK_DIAGONAL, &white);
                        break;

                case BLACK_BISHOP:
                        atk_slide(sq, ATK_DIAGONAL, &black);
                        break;

                case WHITE_KNIGHT:
                        dir = 0;
                        dirs = knight_dirs[sq];
                        do {
                                dir -= dirs;
                                dir &= dirs;
                                to = sq + knight_jump[dir];
                                white.attack[to] += 1;
                        } while (dirs -= dir);
                        break;

                case BLACK_KNIGHT:
                        dir = 0;
                        dirs = knight_dirs[sq];
                        do {
                                dir -= dirs;
                                dir &= dirs;
                                to = sq + knight_jump[dir];
                                black.attack[to] += 1;
                        } while (dirs -= dir);
                        break;

                case WHITE_PAWN:
                        white.pawns[1+F(sq)] += 1;
                        if (F(sq) != FILE_H) {
                                white.attack[sq + DIR_N + DIR_E] += 1;
                        }
                        if (F(sq) != FILE_A) {
                                white.attack[sq + DIR_N - DIR_E] += 1;
                        }
                        break;

                case BLACK_PAWN:
                        black.pawns[1+F(sq)] += 1;
                        if (F(sq) != FILE_H) {
                                black.attack[sq - DIR_N + DIR_E] += 1;
                        }
                        if (F(sq) != FILE_A) {
                                black.attack[sq - DIR_N - DIR_E] += 1;
                        }
                        break;
                }
        }
}

// ---------------------------------------------------------------------------------
void CMSCP::unmake_move(void)
// ---------------------------------------------------------------------------------
{
        int sq;

        for (;;) {
                sq = *--undo_sp;
                if (sq < 0) break;               /* Found sentinel */
                board[sq] = *--undo_sp;
        }
        ply--;
}

// ---------------------------------------------------------------------------------
void CMSCP::make_move(int move)
// ---------------------------------------------------------------------------------
{
        int fr;
        int to;
        int sq;

        *undo_sp++ = -1;                        /* Place sentinel */

        if (board[EP]) {                        /* Clear en-passant info */
                *undo_sp++ = board[EP];
                *undo_sp++ = EP;
                board[EP] = 0;
        }

        to = TO(move);
        fr = FR(move);

        if (move & SPECIAL) {                   /* Special moves first */
                switch (R(fr)) {
                case RANK_8:                    /* Black castles */
                        unmake_move();
                        if (to == G8) {
                                make_move(MOVE(H8,F8));
                        } else {
                                make_move(MOVE(A8,D8));
                        }
                        break;

                case RANK_7:
                        if (board[fr] == BLACK_PAWN) { /* Set en-passant flag */
                                *undo_sp++ = 0;
                                *undo_sp++ = EP;
                                board[EP] = to;
                        } else {                /* White promotes */
                                *undo_sp++ = board[fr];
                                *undo_sp++ = fr;
                                board[fr] = WHITE_QUEEN + (move>>13);
                        }
                        break;

                case RANK_5:                    /* White captures en-passant */
                case RANK_4:                    /* Black captures en-passant */
                        sq = SQ(F(to),R(fr));
                        *undo_sp++ = board[sq];
                        *undo_sp++ = sq;
                        board[sq] = EMPTY;
                        break;

                case RANK_2:
                        if (board[fr] == WHITE_PAWN) { /* Set en-passant flag */
                                *undo_sp++ = 0;
                                *undo_sp++ = EP;
                                board[EP] = to;
                        } else {                /* Black promotes */
                                *undo_sp++ = board[fr];
                                *undo_sp++ = fr;
                                board[fr] = BLACK_QUEEN + (move>>13);
                        }
                        break;

                case RANK_1:                    /* White castles */
                        unmake_move();
                        if (to == G1) {
                                make_move(MOVE(H1,F1));
                        } else {
                                make_move(MOVE(A1,D1));
                        }
                        break;

                default:
                        break;
                }
        }

        ply++;
        if (board[to]!=EMPTY ||
            board[fr]==WHITE_PAWN || board[fr]==BLACK_PAWN
        ) {
                *undo_sp++ = board[LAST];
                *undo_sp++ = LAST;
                board[LAST] = ply;
        }

        *undo_sp++ = board[to];
        *undo_sp++ = to;
        *undo_sp++ = board[fr];
        *undo_sp++ = fr;

        board[to] = board[fr];
        board[fr] = EMPTY;

        if (board[CASTLE] & (castle[fr] | castle[to])) {
                *undo_sp++ = board[CASTLE];
                *undo_sp++ = CASTLE;
                board[CASTLE] &= ~(castle[fr] | castle[to]);
        }
}

// ---------------------------------------------------------------------------------
int CMSCP::push_move(int fr, int to)
// ---------------------------------------------------------------------------------
{
        unsigned short prescore = PRESCORE_EQUAL;
        int move;

        /* what do we capture */
        if (board[to] != EMPTY) {
                prescore += (1<<9) + prescore_piece_value[board[to]];
        }

        /* does the destination square look safe? */
        if (WTM) {
                if (black.attack[to] != 0) { /* defended */
                        prescore -= prescore_piece_value[board[fr]];
                }
        } else {
                if (white.attack[to] != 0) { /* defended */
                        prescore -= prescore_piece_value[board[fr]];
                }
        }

        if (prescore >= caps) {
                move = MOVE(fr, to);
                move_sp->imove = move;
                move_sp->prescore = prescore | history[move];
                move_sp++;
                return 1;
        }
        return 0;
}

// ---------------------------------------------------------------------------------
void CMSCP::push_special_move(int fr, int to)
// ---------------------------------------------------------------------------------
{
        int move;

        move = MOVE(fr, to);
        move_sp->prescore = PRESCORE_EQUAL | history[move];
        move_sp->imove = move | SPECIAL;
        move_sp++;
}

// ---------------------------------------------------------------------------------
void CMSCP::push_pawn_move(int fr, int to)
// ---------------------------------------------------------------------------------
{
        if ((R(to) == RANK_8) || (R(to) == RANK_1)) {
                push_special_move(fr, to);          /* queen promotion */
                push_special_move(fr, to);          /* rook promotion */
                move_sp[-1].imove += 1<<13;
                push_special_move(fr, to);          /* bishop promotion */
                move_sp[-1].imove += 2<<13;
                push_special_move(fr, to);          /* knight promotion */
                move_sp[-1].imove += 3<<13;
        } else {
                push_move(fr, to);
        }
}

// ---------------------------------------------------------------------------------
void CMSCP::gen_slides(int fr, byte dirs)
// ---------------------------------------------------------------------------------
{
        int vector;
        int to;
        byte dir = 0;

        dirs &= king_dirs[fr];
        do {
                dir -= dirs;
                dir &= dirs;
                vector = king_step[dir];
                to = fr;
                do {
                        to += vector;
                        if (board[to] != EMPTY) {
                                if (PIECE_COLOR(board[to]) != WTM) {
                                        push_move(fr, to);
                                }
                                break;
                        }
                        push_move(fr, to);
                } while (dir & king_dirs[to]);
        } while (dirs -= dir);
}

// ---------------------------------------------------------------------------------
int CMSCP::test_illegal(int move)
// ---------------------------------------------------------------------------------
{
        make_move(move);
        compute_attacks();
        unmake_move();
        return myfriend->attack[enemy->king] != 0;
}

// ---------------------------------------------------------------------------------
void CMSCP::generate_moves(unsigned treshold)
// ---------------------------------------------------------------------------------
{
        int             fr, to;
        int             pc;
        byte            dir, dirs;

        caps = treshold;

        for (fr=0; fr<64; fr++) {
                pc = board[fr];
                if (!pc || PIECE_COLOR(pc) != WTM) continue;

                /*
                 *  generate moves for this piece
                 */
                switch (pc) {
                case WHITE_KING:
                case BLACK_KING:
                        dir = 0;
                        dirs = king_dirs[fr];
                        do {
                                dir -= dirs;
                                dir &= dirs;
                                to = fr+king_step[dir];
                                if (board[to] != EMPTY &&
                                    PIECE_COLOR(board[to]) == WTM) continue;
                                push_move(fr, to);
                        } while (dirs -= dir);
                        break;

                case WHITE_QUEEN:
                case BLACK_QUEEN:
                        gen_slides(fr, ATK_SLIDER);
                        break;

                case WHITE_ROOK:
                case BLACK_ROOK:
                        gen_slides(fr, ATK_ORTHOGONAL);
                        break;

                case WHITE_BISHOP:
                case BLACK_BISHOP:
                        gen_slides(fr, ATK_DIAGONAL);
                        break;

                case WHITE_KNIGHT:
                case BLACK_KNIGHT:
                        dir = 0;
                        dirs = knight_dirs[fr];
                        do {
                                dir -= dirs;
                                dir &= dirs;
                                to = fr+knight_jump[dir];
                                if (board[to] != EMPTY &&
                                    PIECE_COLOR(board[to]) == WTM) continue;
                                push_move(fr, to);
                        } while (dirs -= dir);
                        break;

                case WHITE_PAWN:
                        if (F(fr) != FILE_H) {
                                to = fr + DIR_N + DIR_E;
                                if (board[to] >= BLACK_KING) {
                                        push_pawn_move(fr, to);
                                }
                        }
                        if (F(fr) != FILE_A) {
                                to = fr + DIR_N - DIR_E;
                                if (board[to] >= BLACK_KING) {
                                        push_pawn_move(fr, to);
                                }
                        }
                        to = fr + DIR_N;
                        if (board[to] != EMPTY) {
                                break;
                        }
                        push_pawn_move(fr, to);
                        if (R(fr) == RANK_2) {
                                to += DIR_N;
                                if (board[to] == EMPTY) {
                                        if (push_move(fr, to))
                                        if (black.attack[to-DIR_N]) {
                                                move_sp[-1].imove |= SPECIAL;
                                        }
                                }
                        }
                        break;

                case BLACK_PAWN:
                        if (F(fr) != FILE_H) {
                                to = fr - DIR_N + DIR_E;
                                if (board[to] && board[to] < BLACK_KING) {
                                        push_pawn_move(fr, to);
                                }
                        }
                        if (F(fr) != FILE_A) {
                                to = fr - DIR_N - DIR_E;
                                if (board[to] && board[to] < BLACK_KING) {
                                        push_pawn_move(fr, to);
                                }
                        }
                        to = fr - DIR_N;
                        if (board[to] != EMPTY) {
                                break;
                        }
                        push_pawn_move(fr, to);
                        if (R(fr) == RANK_7) {
                                to -= DIR_N;
                                if (board[to] == EMPTY) {
                                        if (push_move(fr, to))
                                        if (white.attack[to+DIR_N]) {
                                                move_sp[-1].imove |= SPECIAL;
                                        }
                                }
                        }
                        break;
                }
        }

        /*
         *  generate castling moves
         */
        if (board[CASTLE] && !enemy->attack[myfriend->king]) {
                if (WTM && (board[CASTLE] & CASTLE_WHITE_KING) &&
                        !board[F1] && !board[G1] &&
                        !enemy->attack[F1])
                {
                        push_special_move(E1, G1);
                }
                if (WTM && (board[CASTLE] & CASTLE_WHITE_QUEEN) &&
                        !board[D1] && !board[C1] && !board[B1] &&
                        !enemy->attack[D1])
                {
                        push_special_move(E1, C1);
                }
                if (!WTM && (board[CASTLE] & CASTLE_BLACK_KING) &&
                        !board[F8] && !board[G8] &&
                        !enemy->attack[F8])
                {
                        push_special_move(E8, G8);
                }
                if (!WTM && (board[CASTLE] & CASTLE_BLACK_QUEEN) &&
                        !board[D8] && !board[C8] && !board[B8] &&
                        !enemy->attack[D8])
                {
                        push_special_move(E8, C8);
                }
        }

        /*
         *  generate en-passant captures
         */
        if (board[EP]) {
                int ep = board[EP];

                if (WTM) {
                        if (F(ep) != FILE_A && board[ep-DIR_E] == WHITE_PAWN) {
                                if (push_move(ep-DIR_E, ep+DIR_N))
                                        move_sp[-1].imove |= SPECIAL;
                        }
                        if (F(ep) != FILE_H && board[ep+DIR_E] == WHITE_PAWN) {
                                if (push_move(ep+DIR_E, ep+DIR_N))
                                        move_sp[-1].imove |= SPECIAL;
                        }
                } else {
                        if (F(ep) != FILE_A && board[ep-DIR_E] == BLACK_PAWN) {
                                if (push_move(ep-DIR_E, ep-DIR_N))
                                        move_sp[-1].imove |= SPECIAL;
                        }
                        if (F(ep) != FILE_H && board[ep+DIR_E] == BLACK_PAWN) {
                                if (push_move(ep+DIR_E, ep-DIR_N))
                                        move_sp[-1].imove |= SPECIAL;
                        }
                }
        }
}

// ---------------------------------------------------------------------------------
int CMSCP::parse_move(char *line, int *num)
// ---------------------------------------------------------------------------------
{
        int                     move, matches;
        int                     n = 0;
        struct move             *m;
        char                    *piece = NULL;
        char                    *fr_file = NULL;
        char                    *fr_rank = NULL;
        char                    *to_file = NULL;
        char                    *to_rank = NULL;
        char                    *prom_piece = NULL;
        char                    *s;

        while (xisspace(line[n]))      /* skip white space */
                n++;

        if (!strncmp(line+n, "o-o-o", 5)
        ||  !strncmp(line+n, "O-O-O", 5)
        ||  !strncmp(line+n, "0-0-0", 5)) {
                piece = "K"; fr_file = "e"; to_file = "c";
                n+=5;
        }
        else if (!strncmp(line+n, "o-o", 3)
        ||  !strncmp(line+n, "O-O", 3)
        ||  !strncmp(line+n, "0-0", 3)) {
                piece = "K"; fr_file = "e"; to_file = "g";
                n+=3;
        }
        else {
                s = strchr("KQRBNP", line[n]);
                if (s && *s) {
                        piece = s;
                        n++;
                }

                /* first square */

                s = strchr("abcdefgh", line[n]);
                if (s && *s) {
                        to_file = s;
                        n++;
                }

                s = strchr("12345678", line[n]);
                if (s && *s) {
                        to_rank = s;
                        n++;
                }

                if (line[n] == '-' || line[n] == 'x') {
                        n++;
                }

                s = strchr("abcdefgh", line[n]);
                if (s && *s) {
                        fr_file = to_file;
                        fr_rank = to_rank;
                        to_file = s;
                        to_rank = NULL;
                        n++;
                }

                s = strchr("12345678", line[n]);
                if (s && *s) {
                        to_rank = s;
                        n++;
                }

                if (line[n] == '=') {
                        n++;
                }
                s = strchr("QRBNqrbn", line[n]);
                if (s && *s) {
                        prom_piece = s;
                        n++;
                }
        }

        while (line[n] == '+' || line[n] == '#'
        || line[n] == '!' || line[n] == '?') {
                n++;
        }

        *num = n;
        if (!piece && !fr_file && !fr_rank
        && !to_file && !to_rank && !prom_piece)
                return 0;

        move = 0;
        matches = 0;

        m = move_sp;
        compute_attacks();
        generate_moves(0);
        while (move_sp > m) {
                int fr, to;

                move_sp--;

                fr = FR(move_sp->imove);
                to = TO(move_sp->imove);

                /* does this move match? */

                if ((piece && *piece != toupper(PIECE2CHAR(board[fr])))
                 || (to_file && *to_file != FILE2CHAR(F(to)))
                 || (to_rank && *to_rank != RANK2CHAR(R(to)))
                 || (fr_file && *fr_file != FILE2CHAR(F(fr)))
                 || (fr_rank && *fr_rank != RANK2CHAR(R(fr)))
                 || (prom_piece &&
                        (toupper(*prom_piece) != "QRBN"[(move_sp->imove)>>13])))
                {
                        continue;
                }

                /* if so, is it legal? */
                if (test_illegal(move_sp->imove)) {
                        continue;
                }
                /* probably.. */

                /* do we already have a match? */
                if (move) {
                        int old_is_p, new_is_p;

                        if (piece) {
								GL_WriteDebugOut(L"CMSCP::parse_move ambiguous result(1). ERROR.");    
							    move_sp = m;
                                return 0;
                        }
                        /* if this move is a pawn move and the previously
                           found isn't, we accept it */
                        old_is_p = (board[FR(move)]==WHITE_PAWN) ||
                                (board[FR(move)]==BLACK_PAWN);
                        new_is_p = (board[fr]==WHITE_PAWN) ||
                                (board[fr]==BLACK_PAWN);

                        if (new_is_p && !old_is_p) {
                                move = move_sp->imove;
                                matches = 1;
                        } else if (old_is_p && !new_is_p) {
                        } else if (!old_is_p && !new_is_p)  {
                                matches++;
                        } else if (old_is_p && new_is_p)  {
                                GL_WriteDebugOut(L"CMSCP::parse_move ambiguous result(2). ERROR.");
                                move_sp = m;
                                return 0;
                        }
                } else {
                        move = move_sp->imove;
                        matches = 1;
                }
        }
        if (matches <= 1)
                return move;

        GL_WriteDebugOut(L"CMSCP::parse_move ambiguous result(3). ERROR.");
        return 0;
}

// ---------------------------------------------------------------------------------
std::string CMSCP::print_square(int square)
// ---------------------------------------------------------------------------------
{
    static char sDummy[3];
	memset(&sDummy,0,sizeof(sDummy));
	
	sDummy[0] = FILE2CHAR(F(square));
	sDummy[1] = RANK2CHAR(R(square));

	return(sDummy);
}

// ---------------------------------------------------------------------------------
std::string CMSCP::print_move_long(int move)
// ---------------------------------------------------------------------------------
{
        char sDummy[7];
		memset(sDummy,0,sizeof(sDummy));

		std::string s1 = print_square(FR(move));
		std::string s2 = print_square(TO(move));
	
		sprintf(sDummy,"%s%s",s1.c_str(),s2.c_str());

		std::string sReturn="";
		//if ((board[FR(move)]==WHITE_KING)&&(strcmp(sDummy,"e1g1")==0)) { sReturn="o-o";   return(sReturn); }
		//if ((board[FR(move)]==WHITE_KING)&&(strcmp(sDummy,"e1c1")==0)) { sReturn="o-o-o"; return(sReturn); }
		//if ((board[FR(move)]==BLACK_KING)&&(strcmp(sDummy,"e8g8")==0)) { sReturn="o-o";   return(sReturn); }
		//if ((board[FR(move)]==BLACK_KING)&&(strcmp(sDummy,"e8c8")==0)) { sReturn="o-o-o"; return(sReturn); }

        if (move >= SPECIAL) {
                if ((board[FR(move)] == WHITE_PAWN && R(TO(move)) == RANK_8)
                ||  (board[FR(move)] == BLACK_PAWN && R(TO(move)) == RANK_1)) {
                        sDummy[4] = '=';
                        char* sA = "QRBN";
						sDummy[5] = sA[move >> 13];
                }
        }

		sReturn = sDummy;
		return(sReturn);
}

// ---------------------------------------------------------------------------------
bool CMSCP::setup_board(char *fen)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::setup_board called.");

        int file=FILE_A, rank=RANK_8;

        while (xisspace(*fen)) fen++;

        memset(board, 0, sizeof board);

        while (rank>RANK_1 || file<=FILE_H) {
                int piece = EMPTY;
                int count = 1;

                switch (*fen) {
                case 'K': piece = WHITE_KING; break;
                case 'Q': piece = WHITE_QUEEN; break;
                case 'R': piece = WHITE_ROOK; break;
                case 'B': piece = WHITE_BISHOP; break;
                case 'N': piece = WHITE_KNIGHT; break;
                case 'P': piece = WHITE_PAWN; break;
                case 'k': piece = BLACK_KING; break;
                case 'r': piece = BLACK_ROOK; break;
                case 'q': piece = BLACK_QUEEN; break;
                case 'b': piece = BLACK_BISHOP; break;
                case 'n': piece = BLACK_KNIGHT; break;
                case 'p': piece = BLACK_PAWN; break;
                case '/': rank -= 1; file = FILE_A; fen++; continue;
                case '1': case '2': case '3': case '4':
                case '5': case '6': case '7': case '8':
                        count = *fen - '0';
                        break;
                default:
                        GL_WriteDebugOut(L"CMSCP::setup_board ERROR.");
                        return(false);
                }
                do {
                        board[SQ(file,rank)] = piece;
                        file++;
                } while (--count);
                fen++;
        }
        ply = (fen[1] == 'b');
        board[LAST] = ply;
        fen += 2;

        while (*fen) {
                switch (*fen) {
                case 'K': board[CASTLE] |= CASTLE_WHITE_KING; break;
                case 'Q': board[CASTLE] |= CASTLE_WHITE_QUEEN; break;
                case 'k': board[CASTLE] |= CASTLE_BLACK_KING; break;
                case 'q': board[CASTLE] |= CASTLE_BLACK_QUEEN; break;
                case 'a': case 'b': case 'c': case 'd':
                case 'e': case 'f': case 'g': case 'h':
                        board[EP] = SQ(CHAR2FILE(*fen),WTM?RANK_5:RANK_4);
                        break;
                default:
                        break;
                }
                fen++;
        }
        reset();

		return(true);
}

// ---------------------------------------------------------------------------------
void CMSCP::compute_piece_square_tables(void)
// ---------------------------------------------------------------------------------s
{
        int pc, sq;
        int score = 0;
        int file;

        compute_attacks(); /* so we know where the king and pawns are */

        for (sq=0; sq<64; sq++) {
                file = 1+F(sq);
                for (pc=1; pc<13; pc++) {
                        switch (pc) {
                        case WHITE_KNIGHT:
                                score = 300;
                                score += center[sq];
                                score -= TAXI(black.king, sq);
                                break;

                        case BLACK_KNIGHT:
                                score = -300;
                                score -= center[sq];
                                score += TAXI(white.king, sq);
                                break;

                        case WHITE_BISHOP:
                                score = +300;
                                break;

                        case BLACK_BISHOP:
                                score = -300;
                                break;

                        case WHITE_ROOK:
                                score = +500;
                                score += DIFF(F(black.king), F(sq));
                                break;

                        case BLACK_ROOK:
                                score = -500;
                                score -= DIFF(F(white.king), F(sq));
                                break;

                        case WHITE_QUEEN:
                                score = +900;
                                score += center[sq];
                                break;

                        case BLACK_QUEEN:
                                score = -900;
                                score -= center[sq];
                                break;

                        case WHITE_KING:
                                score = +400;
                                score += center[sq];
                                break;

                        case BLACK_KING:
                                score = -400;
                                score -= center[sq];
                                break;

                        case WHITE_PAWN:
                                score = +100;
                                score += pawn_advance[sq];
                                if (!black.pawns[file]) {
                                        score += pawn_advance[sq];
                                }
                                break;

                        case BLACK_PAWN:
                                score = -100;
                                score -= pawn_advance[FLIP(sq)];
                                if (!white.pawns[file]) {
                                        score -= pawn_advance[FLIP(sq)];
                                }
                                break;
                        }
                        piece_square[pc-1][sq] = score;
                }
        }
        piece_square[WHITE_KING-1][G1] += 100;
        piece_square[BLACK_KING-1][G8] -= 100;
}

// ---------------------------------------------------------------------------------
int CMSCP::evaluate(void)
// ---------------------------------------------------------------------------------
{
        int sq;
        int score = 0;

        int white_has, black_has;

        /*
         *  stage 1: material+piece_square tables
         */
        for (sq=0; sq<64; sq++) {
                int file;

                if (board[sq] == EMPTY) continue;
                score += piece_square[board[sq]-1][sq];

                file = F(sq)+1;
                switch (board[sq]) {
                case WHITE_PAWN:
                {
                        int missing;
                        if (white.pawns[file] > 1) {
                                score -= 15;
                        }
                        missing = !white.pawns[file-1] + !white.pawns[file+1] +
                                !black.pawns[file];
                        score -= missing * missing * 5;
                        break;
                }
                case BLACK_PAWN:
                {
                        int missing;
                        if (black.pawns[file] > 1) {
                                score += 15;
                        }
                        missing = !black.pawns[file-1] + !black.pawns[file+1] +
                                !white.pawns[file];
                        score += missing * missing * 5;
                        break;
                }
                case WHITE_ROOK:
                        if (!white.pawns[file]) {
                                score += 10;
                                if (!black.pawns[file]) {
                                        score += 10;
                                }
                        }
                        break;

                case BLACK_ROOK:
                        if (!black.pawns[file]) {
                                score -= 10;
                                if (!white.pawns[file]) {
                                        score -= 10;
                                }
                        }
                        break;

                default:
                        break;
                }
        }

        /*
         *  stage 2: board control
         */
        white_has = 0;
        black_has = 0;
        for (sq=0; sq<64; sq++) {
                if (white.attack[sq] > black.attack[sq]) {
                        white_has += white_control[sq];
                }
                if (white.attack[sq] < black.attack[sq]) {
                        black_has += white_control[FLIP(sq)];
                }
        }
        score += (white_has - black_has);

        /* some noise to randomize play */
        score += (hash_stack[ply] ^ rnd_seed) % 17 - 8;

        return WTM ? score : -score;
}

// ---------------------------------------------------------------------------------
int CMSCP::qsearch(int alpha, int beta)
// ---------------------------------------------------------------------------------
{
        int                             best_score;
        int                             score;
        struct move                     *moves;

        nodes++;

        hash_stack[ply] = compute_hash();
        best_score = evaluate();
        if (best_score >= beta) {
                return best_score;
        }

        moves = move_sp;
        generate_moves(PRESCORE_EQUAL + (1<<9));
        qsort(moves, move_sp - moves, sizeof(*moves), cmp_move);
        while (move_sp > moves) {

				if (m_bAbortCalculation) return(0);

                int move;

                move_sp--;
                move = move_sp->imove;
                make_move(move);

                compute_attacks();
                if (myfriend->attack[enemy->king]) {
                        unmake_move();
                        continue;
                }

                score = - qsearch(-beta, -alpha);

                unmake_move();

                if (score <= best_score) {
                        continue;
                }
                best_score = score;

                if (score <= alpha) {
                        continue;
                }
                alpha = score;

                if (score < beta) {
                        continue;
                }
                move_sp = moves; /* fail high: skip remaining moves */
        }
        return best_score;
}

// ---------------------------------------------------------------------------------
int CMSCP::search(int depth, int alpha, int beta)
// ---------------------------------------------------------------------------------
{
        int                             best_score = -INF;
        int                             best_move = 0;
        int                             score;
        struct move                     *moves;
        int                             incheck = 0;
		struct TTBK::TT                 *tt;
        int                             oldalpha = alpha;
        int                             oldbeta = beta;
        int                             i, count=0;

        nodes++;

        /* test for draw by repetition */
		hash_stack[ply] = compute_hash();  
        for (i=ply-4; i>=board[LAST]; i-=2) {
                if (hash_stack[i] == hash_stack[ply]) count++;
                if (count>=2) return 0;
        }

        /*
         *  check transposition table
         */
        tt = &TTABLE[ ((hash_stack[ply]>>16) & (CORE-1)) ];
        if (tt->hash == (hash_stack[ply] & 0xffffU)) {
                if (tt->depth >= depth) {
                        if (tt->flag >= 0) alpha = MAX(alpha, tt->score);
                        if (tt->flag <= 0) beta = MIN(beta,  tt->score);
                        if (alpha >= beta) return tt->score;
                }
                best_move = tt->move & 07777;
        }

        history[best_move] |= PRESCORE_HASHMOVE;
        incheck = enemy->attack[myfriend->king];

        /*
         *  generate moves
         */
        moves = move_sp;
        generate_moves(0);

        history[best_move] &= 0x3fff;
        best_move = 0;

        qsort(moves, move_sp - moves, sizeof(*moves), cmp_move);

        /*
         *  loop over all moves
         */
        while (move_sp > moves) {

				if (m_bAbortCalculation) return(0);

                int newdepth;
                int move;
                move_sp--;
                move = move_sp->imove;
                make_move(move);
                compute_attacks();
                if (myfriend->attack[enemy->king]) {
                        unmake_move();
                        continue;
                }

                newdepth = incheck ? depth : depth-1;
                if (newdepth <= 0) {
                        score = -qsearch(-beta, -alpha);
                } else {
                        score = -search(newdepth, -beta, -alpha);
                }
                if (score < -29000) score++;    /* adjust for mate-in-n */

                unmake_move();

                if (score <= best_score) continue;
                best_score = score;
                best_move = move;

                if (score <= alpha) continue;
                alpha = score;

                if (score < beta) continue;

                move_sp = moves; /* fail high: skip remaining moves */
        }

        if (best_score == -INF) { /* deal with mate and stalemate */
                if (incheck) {
                        best_score = -30000;
                } else {
                        best_score = 0;
                }
        }

        history[best_move & 07777] += depth*depth;
        if (history[best_move & 07777] > 511) {
                int m;
                for (m=0; m<SPECIAL; m++) {
                        history[m] >>= 4;       /* scaling */
                }
        }

        tt->hash = hash_stack[ply] & 0xffffU;
        tt->move = best_move;
        tt->score = best_score;
        tt->depth = depth;
        tt->flag = (oldalpha < best_score) - (best_score < oldbeta);

        return best_score;
}

/* squeeze: compacts a long into a short */
// ---------------------------------------------------------------------------------
unsigned short CMSCP::squeeze(unsigned long n)
// ---------------------------------------------------------------------------------
{
        const unsigned long mask = ~0U<<11;
        unsigned short s;

        for (s=0; n&mask; n>>=1, s-=mask)
                /* SKIP */;

        return s | n;
}

// ---------------------------------------------------------------------------------
int CMSCP::root_search(int maxdepth)
// ---------------------------------------------------------------------------------
{
        int             depth;
        int             score, best_score;
        int             move = 0;
        int             alpha, beta;
        unsigned long   node;
        struct move     *m;

		// Debug: Search time
		DWORD dwStartTime = ::GetTickCount();

        nodes = 0;
        compute_piece_square_tables();

		generate_moves(0);
        qsort(move_stack, move_sp-move_stack, sizeof(struct move), cmp_move);

		alpha = -INF;
        beta = +INF;

        for (depth = 1; depth <= maxdepth; depth++) {
                m = move_stack;
                best_score = INT_MIN;

                node = nodes;
                while (m < move_sp)
				{
                        make_move(m->imove);
                        compute_attacks();
                        
						if (myfriend->attack[enemy->king] != 0) { /* illegal? */
                                unmake_move();
                                *m = *--move_sp; /* drop this move */
                                continue;
                        }
                       
                        hash_stack[ply] = compute_hash();

                        if (depth-1 > 0)
						{
							score = -search(depth-1, -beta, -alpha);
                        } else
						{
							score = -qsearch(-beta, -alpha);
                        }
                        unmake_move();

						// If abort flag set, return "nomove"
						if (m_bAbortCalculation) return(0);

                        if (score>=beta || (score<=alpha && m==move_stack)) {
                                alpha = -INF;
                                beta = +INF;
                                continue; /* re-search this move */
                        }

                        m->prescore = ~squeeze(nodes-node);
                        node = nodes;

                        if (score > best_score) {
                                struct move tmp;

                                best_score = score;
                                alpha = score;
                                beta = score + 1;
                                move = m->imove;

                                tmp = *move_stack; /* swap with top of list */
                                *move_stack = *m;
                                *m = tmp;
                        }
                        m++; /* continue with next move */
                }

                if (move_sp-move_stack <= 1) {
                        break; /* just one move to play */
                }

				/* sort remaining moves in descending order of subtree size */
                qsort(move_stack+1, move_sp-move_stack-1, sizeof(*m), cmp_move);
                alpha = best_score - 33;        /* aspiration window */
                beta = best_score + 33;

				// Abort condition: Flag set
				if (m_bAbortCalculation) break;
        }
        move_sp = move_stack;

		// Debug: Search time
		DWORD dwDeltaTime = ::GetTickCount()-dwStartTime;
		GL_WriteDebugOut(L"CMSCP::root_search time: %.2f sec.",(float)(dwDeltaTime/1000.0f));

        return move;
}

// ---------------------------------------------------------------------------------
bool CMSCP::NewGame(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::NewGame called.");

	if (m_bIsCalculating) AbortThinking(); // Something was running
	
	// ForsythEdwards Notation (FEN)
	// http://en.wikipedia.org/wiki/Forsyth%E2%80%93Edwards_Notation
	
    ComputerPlaysNone();
    rnd_seed = time(NULL);

	return(setup_board("rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -"));
}

// ---------------------------------------------------------------------------------
void CMSCP::ComputerPlaysBoth(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::ComputerPlaysBoth called.");
	computer[0] = 1;
    computer[1] = 1;
}

// ---------------------------------------------------------------------------------
void CMSCP::ComputerPlaysNone(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::ComputerPlaysNone called.");
	computer[0] = 0;
    computer[1] = 0;
}

// ---------------------------------------------------------------------------------
void CMSCP::ComputerPlaysWhite(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::ComputerPlaysWhite called.");
	computer[0] = 0;
    computer[1] = 1;
    ply = 0;
    reset();
}

// ---------------------------------------------------------------------------------
void CMSCP::ComputerPlaysBlack(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::ComputerPlaysBlack called.");
	computer[0] = 1;
    computer[1] = 0;
    ply = 1;
    reset();
}

// ---------------------------------------------------------------------------------
void CMSCP::SetMaxSearchDepth(int iDepth)
// ---------------------------------------------------------------------------------
{
	if (m_bIsCalculating) return; // Something already running

	maxdepth = MAX(1, MIN(iDepth,8));

	GL_WriteDebugOut(L"CMSCP::SetMaxSearchDepth set: %i.",maxdepth);
}

// ---------------------------------------------------------------------------------
bool CMSCP::Go(std::wstring wcsCallbackFunction)
// ---------------------------------------------------------------------------------
{
	//GL_WriteDebugOut(L"CMSCP::Go called");

	if (m_bIsCalculating) return(false); // Something already running

	computer[!WTM] = 1;
    computer[WTM] = 0;
	
	// Set marker that a thread is running
	m_bIsCalculating = true;

	// Reset the flag that lets the calculation terminate early
	m_bAbortCalculation = false;

	// Remember the function name to call after finishing calculation
	m_wcsCallbackFunction = wcsCallbackFunction;

	// There should be no thread running, maybe just a thread handle left over
	if (m_hThreadHandle) 
	{
		GL_WriteDebugOut(L"CMSCP::Go clearing m_hThreadHandle");
		WaitForSingleObject(m_hThreadHandle,INFINITE);
		CloseHandle(m_hThreadHandle);
		m_hThreadHandle = NULL;
	}
	
	// Start new calculation thread here
	m_hThreadHandle = CreateThread( 
									NULL,             // default security attributes
									0,                // use default stack size  
									MyThreadFunction1,// thread function name
									this,			  // argument to thread function 
									0,                // use default creation flags 
									NULL);            // returns the thread identifier 

	// If CreateThread failed, reset the flag
	m_bIsCalculating = (m_hThreadHandle!=NULL);
	GL_WriteDebugOut(L"CMSCP::Go m_bIsCalculating: %i",m_bIsCalculating);

	return(m_bIsCalculating);
}

// ---------------------------------------------------------------------------------
std::string CMSCP::Update1(void)
// ---------------------------------------------------------------------------------
{
	//GL_WriteDebugOut(L"CMSCP::Update1 called.");

	std::string sReturn="";   

	if (computer[!WTM])
	{
		memset(&core, 0, sizeof(core));
        memset(history, 0, sizeof(history));
        int move = root_search(maxdepth);

		if (CheckForDrawByRepetition())
		{
			ComputerPlaysNone();
			sReturn="1/2-1/2 Draw by repetition";
            return(sReturn);
		}

		if (ply >= 1000)
		{
			ComputerPlaysNone();
			sReturn="1/2-1/2 Maximum moves reached";
            return(sReturn);
		}

		if (!move)
		{			   
			   compute_attacks();
               if (!move && enemy->attack[myfriend->king] != 0) (WTM ? sReturn="0-1" : sReturn="1-0");
               else if (!move) sReturn="1/2-1/2 Stale mate";
			   else MessageBox(NULL,L"ERROR on final game state",NULL,MB_OK); 	   
				             
			   ComputerPlaysNone();
               return(sReturn);
		 }
	
		sReturn = print_move_long(move);
        make_move(move);
        hash_stack[ply] = compute_hash();
   }

   return(sReturn);
}

// ---------------------------------------------------------------------------------
std::string CMSCP::Update2(void)
// ---------------------------------------------------------------------------------
{
	//GL_WriteDebugOut(L"CMSCP::Update2 called.");

	std::string sReturn="";   

	if (computer[!WTM])
	{
		memset(&core, 0, sizeof(core));
        memset(history, 0, sizeof(history));
        int move = root_search(maxdepth);

		if (CheckForDrawByRepetition())
		{
			ComputerPlaysNone();
			sReturn="1/2-1/2 Draw by repetition";
            return(sReturn);
		}

		if (ply >= 1000)
		{
			ComputerPlaysNone();
			sReturn="1/2-1/2 Maximum moves reached";
            return(sReturn);
		}

		if (!move)
		{			   
			   compute_attacks();
               if (!move && enemy->attack[myfriend->king] != 0) (WTM ? sReturn="0-1" : sReturn="1-0");
               else if (!move) sReturn="1/2-1/2 Stale mate";
			   else MessageBox(NULL,L"ERROR on final game state",NULL,MB_OK); 	   
				             
			   ComputerPlaysNone();
               return(sReturn);
		 }
	
		sReturn = print_move_long(move);
        //make_move(move);
        //hash_stack[ply] = compute_hash();
   }

   return(sReturn);
}

// ---------------------------------------------------------------------------------
CHESSCOLOR CMSCP::GetCurrentPlayer(void)
// ---------------------------------------------------------------------------------
{
	if (WTM) return(WHITE);
	else return(BLACK);
}

// ---------------------------------------------------------------------------------
bool CMSCP::ExecuteMove(char* pMove)
// ---------------------------------------------------------------------------------
{
	if (m_bIsCalculating) return(false); // Something already running

	int move, dummy;

	move = parse_move(pMove, &dummy);
    if (move)
	{
		make_move(move);
        hash_stack[ply] = compute_hash(); 
		return(true);
    }
	else return(false);
}

// ---------------------------------------------------------------------------------
int CMSCP::GetMaxSearchDepth(void)
// ---------------------------------------------------------------------------------
{
	return(maxdepth);
}

// ---------------------------------------------------------------------------------
bool CMSCP::IsValidMove(char* pMove)
// ---------------------------------------------------------------------------------
{
	if (m_bIsCalculating) return(false);
	
	int iDummy;
	int iMove = parse_move(pMove, &iDummy);

	if (iMove) return(true);
	else return(false);
}

// ---------------------------------------------------------------------------------
bool CMSCP::IsKingAttacked()
// ---------------------------------------------------------------------------------
{
	if (m_bIsCalculating) return(false);
	
	compute_attacks();
    if (enemy->attack[myfriend->king] != 0) return(true);
	else return(false);
}

// ---------------------------------------------------------------------------------
std::string CMSCP::GetValidMoves(char* pFrom)
// ---------------------------------------------------------------------------------
{
	char s1[] = "abcdefgh";
	char s2[] = "12345678";
	std::string sReturn="";

	if (m_bIsCalculating) return(sReturn);

	for (int i1=0;i1<8;i1++) 
		for (int i2=0;i2<8;i2++)
		{
			if ((s1[i1]==pFrom[0])&&(s2[i2]==pFrom[1])) continue;

			char sMove[5]; memset(sMove,0,5);

			sMove[0]=pFrom[0]; 
			sMove[1]=pFrom[1];
			sMove[2]=s1[i1];
			sMove[3]=s2[i2]; 

			if (IsValidMove(sMove))
			{
				if (!sReturn.empty()) sReturn.append(","); 
				sReturn.append(sMove);
			}
		}

	// Disabled (miko) as it is implicitely in the possible
	// moves list - and would not consider the FROM position
	//
	// Cover long and short castling 
	// if (IsValidMove("o-o-o"))
	// {		if (!sReturn.empty()) sReturn.append(","); 
	// 		sReturn.append("o-o-o");
	// }

	// if (IsValidMove("o-o"))
	// {		if (!sReturn.empty()) sReturn.append(","); 
	// 		sReturn.append("o-o");
	// }
	
	return(sReturn);
}

// ---------------------------------------------------------------------------------
bool CMSCP::SetPositionsFEN(char* pPositions)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::SetPositionsFEN called.");
	
	// ForsythEdwards Notation (FEN)
	// http://en.wikipedia.org/wiki/Forsyth%E2%80%93Edwards_Notation
	// like, "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -" 

	bool bRet = setup_board(pPositions);
    computer[0] = 0;
    computer[1] = 0;
    rnd_seed = time(NULL);

	return(bRet);
}

// ---------------------------------------------------------------------------------
std::string CMSCP::GetPositionsFEN(void)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::GetPositionsFEN called.");
	
	// ForsythEdwards Notation (FEN)
	// http://en.wikipedia.org/wiki/Forsyth%E2%80%93Edwards_Notation
	// like, "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -" 

	std::string sFENPositions="";

	if (m_bIsCalculating) return(sFENPositions);

	// Step through the tiles, A8->H8...A1->H1 (rank 8->1, file a->h)
	for (int rank=RANK_8; rank>=RANK_1; rank--) 
	{
		int iEmptyCounter=0;
		for (int file=FILE_A; file<=FILE_H; file++) 
		{
			byte pc = board[SQ(file,rank)];
			
			if(pc == EMPTY) iEmptyCounter++;
			else
			{
				if (iEmptyCounter)
				{
					char sDummy[10];
					sprintf(sDummy,"%i",iEmptyCounter);
					sFENPositions.append(sDummy);	
					iEmptyCounter=0;
				}
				sFENPositions.push_back(PIECE2CHAR(pc));
			}			
		}
		
		if (iEmptyCounter)
		{
			char sDummy[10];
			sprintf(sDummy,"%i",iEmptyCounter);
			sFENPositions.append(sDummy);	
			iEmptyCounter=0;
		}

		if (rank!=0) sFENPositions.append("/");
		else sFENPositions.append(" ");
	}

	// Active color
	if (WTM) sFENPositions.append("w ");
	else sFENPositions.append("b ");

	// The castling availability 
	char sDummy[10];
	sprintf(sDummy,"%s%s%s%s",(board[CASTLE] & CASTLE_WHITE_KING ?  "K" : ""),
							  (board[CASTLE] & CASTLE_WHITE_QUEEN ? "Q" : ""),
							  (board[CASTLE] & CASTLE_BLACK_KING ?  "k" : ""),
							  (board[CASTLE] & CASTLE_BLACK_QUEEN ? "q" : ""));
	
	if (strlen(sDummy)==0) sFENPositions.append("-");
	else sFENPositions.append(sDummy);

	sFENPositions.append(" ");

	// En passant things
	if (board[EP]) sFENPositions.append(print_square(board[EP]));
	else sFENPositions.append("-");

	return(sFENPositions);
}

// ---------------------------------------------------------------------------------
bool CMSCP::CheckForDrawByRepetition(void)
// ---------------------------------------------------------------------------------
{
	//GL_WriteDebugOut(L"CMSCP::CheckForDrawByReptition called.");
	int count=0;

	for (int i=ply-4; i>=board[LAST]; i-=2) 
	{
		if (hash_stack[i] == hash_stack[ply]) 
		{
			count++;
			GL_WriteDebugOut(L"EqualHash: %u %u",hash_stack[i], hash_stack[ply]);
		}
		if (count>=2) 
		{
			GL_WriteDebugOut(L"CMSCP::CheckForDrawByRepetition detected draw by repetition.");
			return(true);
		}
	}
	return(false);
}

// ---------------------------------------------------------------------------------
void CMSCP::SetCallbackFunction(std::string sResult)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::SetCallbackFunction called.");
	
	std::wstring wcsMove;

	if (!m_wcsCallbackFunction.empty()&&(!m_bAbortCalculation)) 
	{
		wcsMove.assign(sResult.begin(), sResult.end());
		myclass.SetCallback(m_wcsCallbackFunction, wcsMove);
	}		

	m_wcsCallbackFunction.clear();
	m_bIsCalculating = false;
	m_bAbortCalculation = false;
}

// ---------------------------------------------------------------------------------
int CMSCP::GetCalcRunning()
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::GetCalcRunning called.");
	return(m_bIsCalculating?1:0);
}

// ---------------------------------------------------------------------------------
bool CMSCP::AbortThinking()
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::AbortThinking called.");
	CloseWorkerThread();	
	return(!m_bIsCalculating);
}

// ---------------------------------------------------------------------------------
DWORD WINAPI MyThreadFunction1(LPVOID lpParam)
// ---------------------------------------------------------------------------------
{
	CMSCP* pCMSCP = (CMSCP*) lpParam;

	std::string sResult = pCMSCP->Update1();
	
	pCMSCP->SetCallbackFunction(sResult); // Callback will be executed on next onUpdate()
		
	return(0);
}

// ---------------------------------------------------------------------------------
DWORD WINAPI MyThreadFunction2(LPVOID lpParam)
// ---------------------------------------------------------------------------------
{
	CMSCP* pCMSCP = (CMSCP*) lpParam;

	std::string sResult = pCMSCP->Update2();
	
	pCMSCP->SetCallbackFunction(sResult); // Callback will be executed on next onUpdate()
		
	return(0);
}

// ---------------------------------------------------------------------------------
int CMSCP::GetNumberOfPossibleMoves()
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"CMSCP::GetNumberOfPossibleMoves called.");

	if (m_bIsCalculating) return(-1);
	
	struct move *m;

	m = move_sp;
    compute_attacks();
    generate_moves(0);

	int iCount = 0;
    
	while (move_sp > m)
	{
		move_sp--;
		if (test_illegal(move_sp->imove)) continue;
		
		if (CheckForDrawByRepetition()||(ply >= 1000))
		{
			move_sp = m;
			return(0);
		}        
		
		iCount++;
	}

	return(iCount);
}

// ---------------------------------------------------------------------------------
bool CMSCP::GoProposal(std::wstring wcsCallbackFunction)
// ---------------------------------------------------------------------------------
{
	//GL_WriteDebugOut(L"CMSCP::GoProposal called");

	if (m_bIsCalculating) return(false); // Something already running

	computer[!WTM] = 1;
    computer[WTM] = 0;
	
	// Set marker that a thread is running
	m_bIsCalculating = true;

	// Reset the flag that lets the calculation terminate early
	m_bAbortCalculation = false;

	// Remember the function name to call after finishing calculation
	m_wcsCallbackFunction = wcsCallbackFunction;

	// There should be no thread running, maybe just a thread handle left over
	if (m_hThreadHandle) 
	{
		GL_WriteDebugOut(L"CMSCP::GoProposal clearing m_hThreadHandle");
		WaitForSingleObject(m_hThreadHandle,INFINITE);
		CloseHandle(m_hThreadHandle);
		m_hThreadHandle = NULL;
	}
	
	// Start new calculation thread here
	m_hThreadHandle = CreateThread( 
									NULL,             // default security attributes
									0,                // use default stack size  
									MyThreadFunction2,// thread function name
									this,			  // argument to thread function 
									0,                // use default creation flags 
									NULL);            // returns the thread identifier 

	// If CreateThread failed, reset the flag
	m_bIsCalculating = (m_hThreadHandle!=NULL);
	GL_WriteDebugOut(L"CMSCP::GoProposal m_bIsCalculating: %i",m_bIsCalculating);

	return(m_bIsCalculating);
}