#pragma once

#ifndef STRICT
#define STRICT
#endif
#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#define _CRT_SECURE_NO_WARNINGS // No depricated string warnings please

// Modify the following defines if you have to target a platform prior to the ones specified below.
// Refer to MSDN for the latest info on corresponding values for different platforms.
#ifndef WINVER				// Allow use of features specific to Windows 95 and Windows NT 4 or later.
#define WINVER 0x0500		// Change this to the appropriate value to target Windows 98 and Windows 2000 or later.
#endif

#ifndef _WIN32_WINNT		// Allow use of features specific to Windows NT 4 or later.
#define _WIN32_WINNT 0x0500	// Change this to the appropriate value to target Windows 2000 or later.
#endif						

#ifndef _WIN32_WINDOWS		// Allow use of features specific to Windows 98 or later.
#define _WIN32_WINDOWS 0x0500 // Change this to the appropriate value to target Windows Me or later.
#endif

#ifndef _WIN32_IE			// Allow use of features specific to IE 4.0 or later.
#define _WIN32_IE 0x0500	// Change this to the appropriate value to target IE 5.0 or later.
#endif

#define XP_WIN

#define DIRECTINPUT_VERSION 0x0800
#define _WIN32_DCOM

#define MY_CLASS_NAME "ZDXPLUGINmikochess"
#define MY_DEBUG_NAME L"DXSMiKoChess_Plugin"
#define MY_ID_STRING  L"MiKo's Chess Plugin v1.0.2 / Oct 2010 (GPL) - using code from MSCP/Marcel van Kervinck"

// Used in CMSCP start
#define MAX(a,b) ((a)>(b) ? (a) : (b))
#define MIN(a,b) ((a)<(b) ? (a) : (b))
#define DIFF(a,b) ((a)>(b) ? (a)-(b) : (b)-(a))
#define TAXI(a,b) (DIFF(F(a),F(b)) + DIFF(R(a),R(b)))
#define INF 32000
#define WTM (~ply & 1)  /* White-to-move predicate */
#define CASTLE_WHITE_KING  1
#define CASTLE_WHITE_QUEEN 2
#define CASTLE_BLACK_KING  4
#define CASTLE_BLACK_QUEEN 8
#define PRESCORE_EQUAL       (10U<<9)
#define PRESCORE_HASHMOVE    (3U<<14)
#define PRESCORE_KILLERMOVE  (2U<<14)
#define PRESCORE_COUNTERMOVE (1U<<14)
#define CORE (2048)
#define TTABLE (core.tt)
#define BOOK (core.bk)
#define PIECE_COLOR(pc)			((pc) < BLACK_KING) /* White or black */
#define DIR_N (+1)              /* Offset to step one square up (north) */
#define DIR_E (+8)              /* Offset to step one square right (east) */
#define RANK2CHAR(r)            ('1'+(r))               /* rank to text */
#define CHAR2RANK(c)            ((c)-'1')               /* text to rank */
#define FILE2CHAR(f)            ('a'+(f))               /* file to text */
#define CHAR2FILE(c)            ((c)-'a')               /* text to file */
#define PIECE2CHAR(p)           ("-KQRBNPkqrbnp"[p])    /* piece to text */
#define F(square)               ((square) >> 3)         /* file */
#define R(square)               ((square) & 7)          /* rank */
#define SQ(f,r)                 (((f) << 3) | (r))      /* compose square */
#define FLIP(square)            ((square)^7)            /* flip board */
#define MOVE(fr,to)             (((fr) << 6) | (to))    /* compose move */
#define FR(move)                (((move) & 07700) >> 6) /* from square */
#define TO(move)                ((move) & 00077)        /* target square */
#define SPECIAL                 (1<<12)                 /* for special moves */
#define ATKB_NORTH              0
#define ATKB_NORTHEAST          1
#define ATKB_EAST               2
#define ATKB_SOUTHEAST          3
#define ATKB_SOUTH              4
#define ATKB_SOUTHWEST          5
#define ATKB_WEST               6
#define ATKB_NORTHWEST          7
#define ATK_NORTH               (1 << ATKB_NORTH)
#define ATK_NORTHEAST           (1 << ATKB_NORTHEAST)
#define ATK_EAST                (1 << ATKB_EAST)
#define ATK_SOUTHEAST           (1 << ATKB_SOUTHEAST)
#define ATK_SOUTH               (1 << ATKB_SOUTH)
#define ATK_SOUTHWEST           (1 << ATKB_SOUTHWEST)
#define ATK_WEST                (1 << ATKB_WEST)
#define ATK_NORTHWEST           (1 << ATKB_NORTHWEST)
#define ATK_ORTHOGONAL          ( ATK_NORTH | ATK_SOUTH | \
                                  ATK_WEST  | ATK_EAST  )
#define ATK_DIAGONAL            ( ATK_NORTHEAST | ATK_NORTHWEST | \
                                  ATK_SOUTHEAST | ATK_SOUTHWEST )
#define ATK_SLIDER              ( ATK_ORTHOGONAL | ATK_DIAGONAL )
#define xisspace(c) isspace((int)(c)) 
#define xisalpha(c) isalpha((int)(c))
#define _CONST_RETURN
// CMSCP defines end

// ENUMs, used in CMSCP start
enum {                  /* 64 squares */
        A1, A2, A3, A4, A5, A6, A7, A8,
        B1, B2, B3, B4, B5, B6, B7, B8,
        C1, C2, C3, C4, C5, C6, C7, C8,
        D1, D2, D3, D4, D5, D6, D7, D8,
        E1, E2, E3, E4, E5, E6, E7, E8,
        F1, F2, F3, F4, F5, F6, F7, F8,
        G1, G2, G3, G4, G5, G6, G7, G8,
        H1, H2, H3, H4, H5, H6, H7, H8,
        CASTLE,         /* Castling rights */
        EP,             /* En-passant square */
        LAST            /* Ply number of last capture or pawn push */
		};

	enum {
        EMPTY,
        WHITE_KING, WHITE_QUEEN, WHITE_ROOK,
        WHITE_BISHOP, WHITE_KNIGHT, WHITE_PAWN,
        BLACK_KING, BLACK_QUEEN, BLACK_ROOK,
        BLACK_BISHOP, BLACK_KNIGHT, BLACK_PAWN
	};

	enum { FILE_A, FILE_B, FILE_C, FILE_D, FILE_E, FILE_F, FILE_G, FILE_H };
	enum { RANK_1, RANK_2, RANK_3, RANK_4, RANK_5, RANK_6, RANK_7, RANK_8 };
	enum CHESSCOLOR { NONE, WHITE, BLACK };
// ENUMs, used in CMSCP end

#include <windows.h>

#include "resource.h"

// Various C and Windows includes - change as required

#include <mmsystem.h>
#include <d3d9.h> // Note compile plugins with November 2008 Direct SDK and you don't need to include the d3dx DLL.
#include <d3dx9.h>
#include <dxerr.h>
#include <dxdiag.h>
#include <dxfile.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>
#include <time.h>
#include <initguid.h>
#include <crtdbg.h>
#include <string>
#include <Winbase.h>

// TraceMonkey - It is important you use the version provided with the SDK as they share the same DLL.
#include "../js/src/dist/include/jsapi.h"

// Plugin includes
#include "plugin.h"
#include "MSCP.h"
#include "myclass.h"

